/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassLoaderObjectInputStream.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.marshalling;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.lang.reflect.Proxy;

/**
 * Reader which use the given classloader when recreating the object.
 * @author Florent Benoit
 */
public class ClassLoaderObjectInputStream extends ObjectInputStream {

    /**
     * ClassLoader to use when loading objects.
     */
    private ClassLoader classLoader = null;

    /**
     * This inputstream reader needs an input stream for its constructor.
     * @param inputStream input stream containing bytes
     * @param classLoader specific classloader to use
     * @throws IOException if super class fails
     */
    public ClassLoaderObjectInputStream(final InputStream inputStream, final ClassLoader classLoader) throws IOException {
        super(inputStream);
        this.classLoader = classLoader;
    }

    /**
     * Load the local class equivalent of the specified stream class
     * description.
     * @param desc an instance of class <code>ObjectStreamClass</code>
     * @return a <code>Class</code> object corresponding to <code>desc</code>
     * @throws IOException any of the usual input/output exceptions
     * @throws ClassNotFoundException if class of a serialized object cannot be
     *         found
     */
    @Override
    protected Class<?> resolveClass(final ObjectStreamClass desc) throws IOException, ClassNotFoundException {
        // Use the thread classloader
        return Class.forName(desc.getName(), false, classLoader);
    }

    /**
     * Returns a proxy class that implements the interfaces named in a proxy
     * class descriptor; subclasses may implement this method to read custom
     * data from the stream along with the descriptors for dynamic proxy
     * classes, allowing them to use an alternate loading mechanism for the
     * interfaces and the proxy class.
     * @param interfacesName the list of interface names that were deserialized in
     *        the proxy class descriptor
     * @return a proxy class for the specified interfaces
     * @throws IOException any exception thrown by the underlying
     *         <code>InputStream</code>
     * @throws ClassNotFoundException if the proxy class or any of the named
     *         interfaces could not be found
     * @see java.io.ObjectOutputStream#annotateProxyClass(Class)
     * @since 1.3
     */
    @Override
    protected Class<?> resolveProxyClass(final String[] interfacesName) throws IOException, ClassNotFoundException {
        // Load the interfaces with the
        Class<?>[] interfaces = new Class[interfacesName.length];
        for (int i = 0; i < interfaces.length; i++) {
            interfaces[i] = Class.forName(interfacesName[i], false, classLoader);
        }

        // Use the thread classloader to make the proxy
        try {
            return Proxy.getProxyClass(classLoader, interfaces);
        } catch (IllegalArgumentException e) {
            throw new ClassNotFoundException(null, e);
        }

    }

}
